Cursor

Struct Cursor 

pub struct Cursor<A, L> { /* private fields */ }
Cursor to track log heights (state vector).

It offers methods to “advance” a log and compute the difference to another state vector. A cursor can be used to manage a state vector over a topic (“log heights” of logs scoped by a topic).

Implementations

Source

impl<A, L> Cursor<A, L>
where A: Author, L: LogId,

pub fn new(name: impl AsRef<str>, state: LogHeights<A, L>) -> Self

pub fn name(&self) -> &str

pub fn state(&self) -> &LogHeights<A, L>

Returns state vector.

pub fn log_height(&self, author: &A, log_id: &L) -> Option<&SeqNum>

Returns state vector for a specific log.

pub fn compare(&self, other: &LogHeights<A, L>) -> LogRanges<A, L>

Calculates the difference between two state vectors.

pub fn advance(&mut self, author: A, log_id: L, log_height: SeqNum)

Advances the state of a specific log.