pub trait Extensions:
Clone
+ Debug
+ for<'de> Deserialize<'de>
+ Serialize { }Expand description
Super-trait defining trait bounds required by custom extensions types.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementors
impl<T> Extensions for T
Blanket implementation of Extensions trait any type with the required bounds satisfied.