AnyOperation

Struct AnyOperation 

pub struct AnyOperation {
    pub hash: Hash,
    pub header: AnyHeader,
    pub body: Option<Body>,
}
Combined AnyHeader, Body and operation Hash (Operation Id).

Extensions

AnyOperation does not know the concrete extensions type. On this level it is only concerned with the validity and integrity of the append-only log type itself which is enough for most low-level protocols, such as the sync protocol.

Applications usually want to attach custom extensions to the operation, if you need to know the type you can easily convert from AnyOperation to Operation with an explicit E extensions type.

Fields

hash: Hashheader: AnyHeaderbody: Option<Body>