Builder

Struct Builder 

pub struct Builder<E> { /* private fields */ }
Build & sign operations.

Implementations

Source

impl<E> Builder<E>
where E: Extensions,

pub fn new() -> Self

pub fn body(self, bytes: impl AsRef<[u8]>) -> Self

Attach payload to operation.

pub fn chain(self, seq_num: SeqNum, backlink: Hash) -> Self

Sets the “hash chain” values of this operation: sequence number and backlink.

pub fn seq_num(self, seq_num: SeqNum) -> Self

Number of operations this author has published to this log, begins with 0 and is always incremented by 1 with each new operation by the same author.

Hash of the previous operation of the same author and log. Can be omitted if first operation in log.

pub fn build(self, signing_key: &SigningKey, extensions: E) -> Header<E>

Encodes, signs and returns final header of operation.

A custom header extensions type can be set here as well when required. It will be embedded in the header. Set this to () (unit-type) when extensions are not necessary.