pub struct Header<E = ()> {
pub version: Version,
pub verifying_key: VerifyingKey,
pub signature: Signature,
pub payload_size: PayloadSize,
pub payload_hash: Option<Hash>,
pub seq_num: SeqNum,
pub backlink: Option<Hash>,
pub extensions: E,
/* private fields */
}Expand description
Header of a p2panda operation with known extensions type.
The header holds all metadata required to cryptographically secure and authenticate a message
Body and it’s custom extensions.
See AnyHeader for dealing with headers when you don’t care about the concrete extensions
type (E).
Example
use p2panda_core::{Header, SigningKey};
let signing_key = SigningKey::generate();
let header = Header::builder()
.body(b"Hello, Icebear!")
// Sign the header with the author's private key.
.build(&signing_key, ());Fields
version: VersionOperation format version, allowing backwards compatibility when specification changes.
verifying_key: VerifyingKeyAuthor of this operation.
signature: SignatureSignature by author over all fields in header, providing authenticity.
payload_size: PayloadSizeNumber of bytes of the body of this operation, must be zero if no body is given.
payload_hash: Option<Hash>Hash of the body of this operation, must be included if payload_size is non-zero and omitted otherwise.
Keeping the hash here allows us to delete the payload (off-chain data) while retaining the ability to check the signature of the header.
seq_num: SeqNumNumber of operations this author has published to this log, begins with 0 and is always incremented by 1 with each new operation by the same author.
backlink: Option<Hash>Hash of the previous operation of the same author and log. Can be omitted if first operation in log.
extensions: ECustom additional data.
Implementations
Sourceimpl<E> Header<E>where
E: Extensions,
impl<E> Header<E>where
E: Extensions,
Sourcepub fn decode(bytes: &[u8]) -> Result<Self, HeaderError>
pub fn decode(bytes: &[u8]) -> Result<Self, HeaderError>
Attempts decoding header from bytes.
This might fail if integrity checks failed or header formatting is invalid.