pub struct Cursor<A, L> { /* private fields */ }Expand description
Cursor to track log heights (state vector).
It offers methods to “advance” a log and compute the difference to another state vector. A cursor can be used to manage a state vector over a topic (“log heights” of logs scoped by a topic).
Implementations
Sourceimpl<A, L> Cursor<A, L>
impl<A, L> Cursor<A, L>
pub fn new(name: impl AsRef<str>, state: LogHeights<A, L>) -> Self
pub fn name(&self) -> &str
Sourcepub fn state(&self) -> &LogHeights<A, L>
pub fn state(&self) -> &LogHeights<A, L>
Returns state vector.
Sourcepub fn log_height(&self, author: &A, log_id: &L) -> Option<&SeqNum>
pub fn log_height(&self, author: &A, log_id: &L) -> Option<&SeqNum>
Returns state vector for a specific log.
Sourcepub fn compare(&self, other: &LogHeights<A, L>) -> LogRanges<A, L>
pub fn compare(&self, other: &LogHeights<A, L>) -> LogRanges<A, L>
Calculates the difference between two state vectors.