pub struct AnyOperation {
pub hash: Hash,
pub header: AnyHeader,
pub body: Option<Body>,
}Expand description
Combined AnyHeader, Body and operation Hash (Operation Id).
Extensions
AnyOperation does not know the concrete extensions type. On this level it is only concerned
with the validity and integrity of the append-only log type itself which is enough for most
low-level protocols, such as the sync protocol.
Applications usually want to attach custom extensions to the operation, if you need to know the
type you can easily convert from AnyOperation to Operation with an
explicit E extensions type.
Fields
hash: Hashheader: AnyHeaderbody: Option<Body>Trait Implementations
Sourceimpl Chain<Hash> for AnyOperation
impl Chain<Hash> for AnyOperation
Sourceimpl Clone for AnyOperation
impl Clone for AnyOperation
Sourcefn clone(&self) -> AnyOperation
fn clone(&self) -> AnyOperation
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for AnyOperation
impl Debug for AnyOperation
Sourceimpl Digest<Hash> for AnyOperation
impl Digest<Hash> for AnyOperation
Sourceimpl Offchain<Hash> for AnyOperation
impl Offchain<Hash> for AnyOperation
Sourcefn payload_hash(&self) -> Option<Hash>
fn payload_hash(&self) -> Option<Hash>
Hash digest of the payload.
Sourcefn payload_size(&self) -> PayloadSize
fn payload_size(&self) -> PayloadSize
Size in bytes of the payload.
Sourceimpl Provenance<VerifyingKey> for AnyOperation
impl Provenance<VerifyingKey> for AnyOperation
Sourceimpl TryFrom<(Vec<u8>, Option<Vec<u8>>)> for AnyOperation
impl TryFrom<(Vec<u8>, Option<Vec<u8>>)> for AnyOperation
Sourcetype Error = HeaderError
type Error = HeaderError
The type returned in the event of a conversion error.
Sourceimpl<E> TryFrom<AnyOperation> for Operation<E>where
E: Extensions,
impl<E> TryFrom<AnyOperation> for Operation<E>where
E: Extensions,
Sourcetype Error = HeaderError
type Error = HeaderError
The type returned in the event of a conversion error.